@@ -1179,7 +1179,7 @@ dataproject_on_add_folder(gpointer widget, DataProject *data_project)
     gtk_tree_store_set(data_project->dataproject_compilation_store, &iter,
             DATA_TREE_COL_ICON, icon, DATA_TREE_COL_FILE, _("New Folder"),
             DATA_TREE_COL_SIZE, size, DATA_TREE_COL_HUMANSIZE, human_readable,
-            DATA_TREE_COL_PATH, "", DATA_TREE_COL_STATUS, 0L, -1);
+            DATA_TREE_COL_PATH, "", DATA_TREE_COL_STATUS, STATUS_IS_FOLDER, -1);
 
     g_object_unref(icon);
 
@@ -1526,6 +1526,43 @@ dataproject_on_datadisk_size_changed(GtkOptionMenu *option_menu, DataProject *da
 }
 
 
+/*
+ * Escapes file paths adding '\' before any '\' or '=' character
+ */
+static gchar *
+escape_filepath(gchar *path)
+{
+    GB_LOG_FUNC
+    g_return_val_if_fail(path != NULL, NULL);
+
+    int count = 0;
+    gchar *p = path;
+    gchar *pr = NULL;
+    gchar *result = NULL;
+
+    while (*p != 0) {
+        if (*p == '\\' || *p == '=') {
+            ++count;
+        }
+        ++p;
+    }
+
+    if (count) {
+        result = g_malloc0(strlen(path) + count + 1);
+        p = path;
+        pr = result;
+        while (*p != 0) {
+            if (*p == '\\' || *p == '=') {
+                *pr++ = '\\';
+            }
+            *pr++ = *p++;
+        }
+    } else {
+        result = g_strdup(path);
+    }
+    return result;
+}
+
 /* Adds to the burning list all the contents recursively. This could be improved checking for depth,
  * as mkisofs only accepts a max depth o 6 directories */
 static void
@@ -1557,9 +1594,13 @@ dataproject_build_paths_file_recursive(GtkTreeModel *model, GtkTreeIter *parent_
         else
         {
             gchar *full_path = g_build_filename(file_path, file_name, NULL);
-            fprintf(tmp_file->file_stream, "%s=%s\n", full_path, path_in_system);
+            gchar *escaped_full_path = escape_filepath(full_path);
+            gchar *escaped_path_in_system = escape_filepath(path_in_system);
+            fprintf(tmp_file->file_stream, "%s=%s\n", escaped_full_path, escaped_path_in_system);
             GB_TRACE("dataproject_build_paths_file_recursive - [%s]\n", full_path);
             g_free(full_path);
+            g_free(escaped_full_path);
+            g_free(escaped_path_in_system);
         }
 
         g_free(path_in_system);
@@ -1623,19 +1664,36 @@ dataproject_on_create_datadisk(gpointer widget, DataProject *data_project)
 
     const GBTempFile *tmp_file = dataproject_build_paths_file(GTK_TREE_MODEL(data_project->dataproject_compilation_store));
 
-    if(data_project->is_dvd)
+    if (tmp_file != NULL) 
     {
-        if(data_project->msinfo != NULL)
-            burn_append_data_dvd(tmp_file->file_name, data_project->msinfo);
+        if(data_project->is_dvd)
+        {
+            if(data_project->msinfo != NULL)
+                burn_append_data_dvd(tmp_file->file_name, data_project->msinfo);
+            else
+                burn_create_data_dvd(tmp_file->file_name);
+        }
+        else if(data_project->msinfo != NULL)
+            burn_append_data_cd(tmp_file->file_name, data_project->msinfo);
         else
-            burn_create_data_dvd(tmp_file->file_name);
+            burn_create_data_cd(tmp_file->file_name);
+        /* TODO - we should delete the temp file here */
     }
-    else if(data_project->msinfo != NULL)
-        burn_append_data_cd(tmp_file->file_name, data_project->msinfo);
     else
-        burn_create_data_cd(tmp_file->file_name);
+    {
+      /* TODO - Display an error message */
+        gchar *temp_dir = preferences_get_string(GB_TEMP_DIR);
+        gchar *message = g_strdup_printf(_("Could not create temporary file. \n"
+                                           "Please make sure your temporary "
+                                           "directory [%s] is writable "
+                                           "and try again."), temp_dir);
+        gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
+        g_free(message);
+        g_free(temp_dir);
+        return;
+    }
+
 
-    /* TODO - we should delete the temp file here */
 }
 
 
